
package tcc.payment;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;

import payment.PaymentDbUtils;
import payment.PaymentTccService;

import com.atomikos.icatch.Extent;
import com.atomikos.icatch.ImportingTransactionManager;
import com.atomikos.icatch.Propagation;
import com.atomikos.icatch.config.UserTransactionService;
import com.atomikos.icatch.config.UserTransactionServiceImp;
import com.atomikos.icatch.tcc.UserTccServiceManager;

 /**
  *Copyright &copy; 2002-2006, Atomikos. All rights reserved.
  *
  *An implementation of the payment server.
  *This example shows how to set up TransactionsRMI
  *and how to implement invocations that are to be part
  *of a remote transaction. For simplicity, we use
  *a NonXADataSourceBean class, but if recovery is essential
  *then it is imperative to use XA. See the XA examples
  *on how to do that; the RMI aspects of this demo are
  *completely independent of whether you use XA or not.
  */
  
public class PaymentServerImp
implements PaymentServer
{

	
	private static UserTransactionService uts;
	
	//the TCC service manager to use
    private static UserTccServiceManager tccMgr;
    
    //the service implementation to delegate to
    private static PaymentTccService service;
    
      /**
        *Initialize the TM, and setup DB tables if needed.
        */
        
      private void setup()
      throws Exception
      {
          
            tccMgr = new UserTccServiceManager();
            service = new PaymentTccService ( tccMgr , 1000 );
            //initialize the TCC system with the service to 
            //ensure recovery with the service's help
            tccMgr.registerForRecovery ( service );
            
            uts = new UserTransactionServiceImp();
            uts.init ( uts.createTSInitInfo() );
            
            PaymentDbUtils.createDatabase();
            
            //Lastly, export the server into JNDI
            UnicastRemoteObject.exportObject ( this );
            javax.naming.Context ctx = new javax.naming.InitialContext();
            ctx.rebind ( "PaymentServer" , this );
            
            //That concludes setup
           
      }
      
      /**
        *Shutdown the TM and the datasource.
        */
        
      private  void shutdown()
      throws Exception
      {
          //shutdown and wait for active transactions
          //to finish
          uts.shutdown ( false );

          
          UnicastRemoteObject.unexportObject ( this , false );
          javax.naming.Context ctx = new javax.naming.InitialContext();
          ctx.unbind ( "PaymentServer" );
            
         

      }


     
      
  
      
      private static ImportingTransactionManager 
      getImportingTransactionManager()
      {
          return uts.getImportingTransactionManager(); 
      }
      
      
    /**
      *@see PaymentServer
      */
    
    public Extent pay ( Propagation propagation, String cardno , int amount ) 
    throws PaymentServerException, RemoteException
    {
        //false ASA exception happens
        boolean success = true;
        ImportingTransactionManager imptm = getImportingTransactionManager();
        Extent extent = null;
        
        try {
            //FIRST: import the transaction into the local server.
            //The second argument specifies that orphan detection
            //should be used (an Atomikos added feature that allows
            //extra quality of service). The third argument specifies
            //that timed-out indoubt transactions should be 
            //rolledback (so-called heuristics).
            //After this method returns, the thread will have a local
            //transaction associated to it.
            imptm.importTransaction ( propagation , true , false );
            

            service.tryPayment ( cardno , amount );
        }
        catch ( Exception e ) {
            //e.printStackTrace();
            success = false;
            throw new PaymentServerException ( e.getMessage () );
        }
        finally {
            
            try {
                //VERY IMPORTANT: always indicate to the transaction service
                //whether a call succeeded or not, and get the extent.
                //This finally block is typical for EVERY imported transaction!
                //After this method returns, the executing thread will no longer
                //have any transaction associated to it.
                
               extent = imptm.terminated ( success );
            }
           catch ( com.atomikos.icatch.RollbackException rb ) {
                //happens if transaction timed out
                throw new RemoteException ( "Transaction timed out" );
            }
           
           
        }
        
        //return the extent to the client
        //NOTE: if an exception happens then no extent will be returned,
        //but that is perfectly OK since the local work will not need to be
        //committed in that case!
        return extent;
    }
    
     /**
      *@see PaymentServer
      */
      
    public int getPayments ( String cardno )
    throws PaymentServerException , RemoteException 
    {
        int ret = 0;
        try {
            ret = service.getPayments ( cardno );
        }
        catch ( Exception e ) {
            //e.printStackTrace();
            
            throw new PaymentServerException ( e.getMessage () );
        }
 

        return ret;
    }
    
    
    public static void main ( String[] args ) 
    {
        BufferedReader breader = null;
        
        try {
                           
            PaymentServerImp server = new PaymentServerImp();
            server.setup();
            
            System.out.println ( "Server running..." );
            System.out.println ( "Type ENTER to shutdown." );
            
            breader = 
              new BufferedReader ( new InputStreamReader ( System.in ) );
            breader.readLine();
            
            server.shutdown();
            System.exit ( 0 );
        }
        catch ( Exception e ) {
            e.printStackTrace();
        }
    }
}

